@@ -3,13 +3,16 @@ from __future__ import unicode_literals
 import json
 import logging
 import time
-import urllib2
+import urlparse
 import requests
 
 logger = logging.getLogger(__name__)
 
 class AudioAddict(object):
-    def __init__(self, username, password, quality, difm, radiotunes, rockradio, jazzradio, frescaradio):
+
+    proxies = None
+
+    def __init__(self, username, password, quality, difm, radiotunes, rockradio, jazzradio, frescaradio, proxy=None):
         self._cache = {}
         self._channels = {}
         self._backoff_until = time.time()
@@ -24,6 +27,11 @@ class AudioAddict(object):
         self._jazzradio = jazzradio
         self._frescaradio = frescaradio
 
+        # Set up proxy
+        if proxy is not None:
+            r1 = urlparse.urlsplit(proxy)
+            self.proxies = {r1.scheme: proxy}
+
         # Figure out our API key
         if self._username and self._password:
             self._api_key, self._has_premium = self._fetchApiKey(self._username, self._password)
@@ -122,7 +130,7 @@ class AudioAddict(object):
         logger.info('AudioAddict: authenticating with API')
 
         payload = {'username': username, 'password': password}
-        r = requests.post("https://api.audioaddict.com/v1/di/members/authenticate", data=payload)
+        r = requests.post("https://api.audioaddict.com/v1/di/members/authenticate", data=payload, proxies=self.proxies)
         json_object_raw = r.json()
 
         subscriptions = json_object_raw['subscriptions']
@@ -148,20 +156,29 @@ class AudioAddict(object):
 
         logger.debug('Fetching: %s', uri)
         try:
-            fp = urllib2.urlopen(uri)
-            data = json.load(fp)
+            r = requests.get(uri, proxies=self.proxies)
+            logger.debug("Get %s : %i", uri, r.status_code)
+
+            if r.status_code is not 200:
+                logger.error("AudioAddict: %s is not reachable [http code:%i]", uri, r.status_code)
+                return None
+
+        except requests.exceptions.RequestException, e:
+            logger.error("AudioAddict RequestException: %s", e)
+        except requests.exceptions.ConnectionError, e:
+            logger.error("AudioAddict ConnectionError: %s", e)
+        except requests.exceptions.URLRequired, e:
+            logger.error("AudioAddict URLRequired: %s", e)
+        except requests.exceptions.TooManyRedirects, e:
+            logger.error("AudioAddict TooManyRedirects: %s", e)
+        except Exception, e:
+            logger.error("AudioAddict exception: %s", e)
+        else:
+            # Parse JSON object
+            data = r.json()
             self._cache[uri] = data
             self._backoff = 1
             return data
-        except urllib2.HTTPError as e:
-            logger.debug('Fetch failed, HTTP %s: %s', e.code, e.reason)
-            if e.code == 404:
-                self._cache[uri] = default
-                return default
-        except IOError as e:
-            logger.debug('Fetch failed: %s', e)
-        except ValueError as e:
-            logger.warning('Fetch failed: %s', e)
 
         self._backoff = min(self._backoff_max, self._backoff*2)
         self._backoff_until = time.time() + self._backoff
